@@ -1,7 +1,7 @@
 """
 Core functions used by unumpy and some of its submodules.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 # The functions found in this module cannot be defined in unumpy or
@@ -16,10 +16,11 @@ import sys
 
 # 3rd-party modules:
 import numpy
+from numpy.core import numeric
 
 # Local modules:
 import uncertainties
-from .. import umath
+from uncertainties import umath, deprecation
 
 from uncertainties import __author__
 
@@ -27,7 +28,7 @@ __all__ = [
     # Factory functions:
     'uarray', 'umatrix',
 
-    # Utilties:
+    # Utilities:
     'nominal_values', 'std_devs',
 
     # Classes:
@@ -124,12 +125,15 @@ def wrap_array_func(func):
     func() is given a NumPy array that contains numbers with
     uncertainties.
 
-    func() is supposed to return a NumPy array.
-
     This wrapper is similar to uncertainties.wrap(), except that it
     handles an array argument instead of float arguments.
+
+    However, the returned function is more restricted: the array
+    argument cannot be given as a keyword argument with the name in
+    the original function (it is not a drop-in replacement).
     
-    func -- version that takes and returns a single NumPy array.
+    func -- function whose first argument takes a single NumPy array,
+    and which returns a NumPy array.
     """
 
     @uncertainties.set_doc("""\
@@ -143,10 +147,10 @@ def wrap_array_func(func):
     
     Original documentation:
     %s""" % (func.__name__, func.__doc__))
-    def wrapped_func(arr, *args):
+    def wrapped_func(arr, *args, **kwargs):
         # Nominal value:
         arr_nominal_value = nominal_values(arr)
-        func_nominal_value = func(arr_nominal_value, *args)
+        func_nominal_value = func(arr_nominal_value, *args, **kwargs)
 
         # The algorithm consists in numerically calculating the derivatives
         # of func:
@@ -208,17 +212,18 @@ def wrap_array_func(func):
 
             # Origin value of array arr when var is shifted by shift_var:
             shifted_arr_values = arr_nominal_value + shift_arr
-            func_shifted = func(shifted_arr_values, *args)
+            func_shifted = func(shifted_arr_values, *args, **kwargs)
             numerical_deriv = (func_shifted-func_nominal_value)/shift_var
 
             # Update of the list of variables and associated
             # derivatives, for each element:
-            for (derivative_dict, derivative_value) \
-                  in zip(derivatives.flat, numerical_deriv.flat):
+            for (derivative_dict, derivative_value) in (
+                zip(derivatives.flat, numerical_deriv.flat)):
+                
                 if derivative_value:
                     derivative_dict[var] = derivative_value
 
-        # numbers with uncertainties are build from the result:
+        # numbers with uncertainties are built from the result:
         return numpy.vectorize(uncertainties.AffineScalarFunc)(
             func_nominal_value, derivatives)
 
@@ -232,26 +237,31 @@ def wrap_array_func(func):
 ###############################################################################
 # Arrays
 
-# Vectorized creation of an array of variables:
-
-# ! Looking up uncertainties.Variable beforehand through '_Variable =
-# uncertainties.Variable' does not result in a significant speed up:
-
-_uarray = numpy.vectorize(lambda v, s: uncertainties.Variable(v, s),
-                          otypes=[object])
-
-def uarray((values, std_devs)):
+def uarray(nominal_values, std_devs=None):
     """
     Returns a NumPy array of numbers with uncertainties
     initialized with the given nominal values and standard
     deviations.
 
-    values, std_devs -- valid arguments for numpy.array, with
+    nominal_values, std_devs -- valid arguments for numpy.array, with
     identical shapes (list of numbers, list of lists, numpy.ndarray,
     etc.).
+
+    std_devs=None is only used for supporting legacy code, where
+    nominal_values can be the tuple of nominal values and standard
+    deviations.
     """
 
-    return _uarray(values, std_devs)
+    if std_devs is None:  # Obsolete, single tuple argument call
+        deprecation('uarray() should now be called with two arguments.')
+        (nominal_values, std_devs) = nominal_values
+        
+    return (numpy.vectorize(
+        # ! Looking up uncertainties.Variable beforehand through
+        # '_Variable = uncertainties.Variable' does not result in a
+        # significant speed up:
+        lambda v, s: uncertainties.Variable(v, s), otypes=[object])
+        (nominal_values, std_devs))
 
 ###############################################################################
 
@@ -286,11 +296,12 @@ def func_with_deriv_to_uncert_func(func_with_derivatives):
     and the derivatives of this function with respect to multiple
     scalar parameters are calculated by func_with_derivatives().
     
-    func_with_derivatives(arr, input_type, derivatives, *args) returns
-    an iterator.  The first element is the value of the function at
-    point 'arr' (with the correct type).  The following elements are
-    arrays that represent the derivative of the function for each
-    derivative array from the iterator 'derivatives'.
+    func_with_derivatives(arr, input_type, derivatives, *args,
+    **kwargs) returns an iterator.  The first element is the value of
+    the function at point 'arr' (with the correct type).  The
+    following elements are arrays that represent the derivative of the
+    function for each derivative array from the iterator
+    'derivatives'.
 
       func_with_derivatives takes the following arguments:
 
@@ -311,7 +322,7 @@ def func_with_deriv_to_uncert_func(func_with_derivatives):
     Examples of func_with_derivatives: inv_with_derivatives().
     """
     
-    def wrapped_func(array_like, *args):
+    def wrapped_func(array_like, *args, **kwargs):
         """
         array_like -- array-like object that contains numbers with
         uncertainties (list, Numpy ndarray or matrix, etc.).
@@ -339,7 +350,7 @@ def func_with_deriv_to_uncert_func(func_with_derivatives):
             array_nominal,
             type(array_like),
             (array_derivative(array_version, var) for var in variables),
-            *args)
+            *args, **kwargs)
 
         func_nominal_value = func_and_derivs.next()
 
@@ -411,8 +422,8 @@ def inv_with_derivatives(arr, input_type, derivatives):
         derivative_mat = numpy.asmatrix(derivative)
         yield -inverse_mat * derivative_mat * inverse_mat
 
-_inv = func_with_deriv_to_uncert_func(inv_with_derivatives)
-_inv.__doc__ = """\
+inv = func_with_deriv_to_uncert_func(inv_with_derivatives)
+inv.__doc__ = """\
     Version of numpy.linalg.inv that works with array-like objects
     that contain numbers with uncertainties.
 
@@ -477,11 +488,11 @@ def pinv_with_derivatives(arr, input_type, derivatives, rcond):
 # Default rcond argument for the generalization of numpy.linalg.pinv:
 try:
     # Python 2.6+:
-    _pinv_default = numpy.linalg.pinv.__defaults__[0]
+    pinv_default = numpy.linalg.pinv.__defaults__[0]
 except AttributeError:
-    _pinv_default = 1e-15
+    pinv_default = 1e-15
 
-_pinv_with_uncert = func_with_deriv_to_uncert_func(pinv_with_derivatives)
+pinv_with_uncert = func_with_deriv_to_uncert_func(pinv_with_derivatives)
 
 @uncertainties.set_doc("""
     Version of numpy.linalg.pinv that works with array-like objects
@@ -495,8 +506,8 @@ _pinv_with_uncert = func_with_deriv_to_uncert_func(pinv_with_derivatives)
     Original documentation:
     %s
     """ % numpy.linalg.pinv.__doc__)
-def _pinv(array_like, rcond=_pinv_default):
-    return _pinv_with_uncert(array_like, rcond)
+def pinv(array_like, rcond=pinv_default):
+    return pinv_with_uncert(array_like, rcond)
 
 ########## Matrix class
 
@@ -508,6 +519,17 @@ class matrix(numpy.matrix):
     matrix contains numbers with uncertainties.
     """
 
+    def __rmul__(self, other):
+        # ! NumPy's matrix __rmul__ uses an apparently a restrictive
+        # dot() function that cannot handle the multiplication of a
+        # scalar and of a matrix containing objects (when the
+        # arguments are given in this order).  We go around this
+        # limitation:
+        if numeric.isscalar(other):
+            return numeric.dot(self, other)
+        else:
+            return numeric.dot(other, self)  # The order is important
+
     # The NumPy doc for getI is empty:
     # @uncertainties.set_doc(numpy.matrix.getI.__doc__)
     def getI(self):
@@ -518,9 +540,9 @@ class matrix(numpy.matrix):
 
         M, N = self.shape
         if M == N:
-            func = _inv
+            func = inv
         else:
-            func = _pinv
+            func = pinv
         return func(self)
         
 
@@ -538,18 +560,22 @@ class matrix(numpy.matrix):
     
     std_devs = std_devs
     
-def umatrix(*args):
+def umatrix(nominal_values, std_devs=None):
     """
     Constructs a matrix that contains numbers with uncertainties.
 
-    The input data is the same as for uarray(...): a tuple with the
-    nominal values, and the standard deviations.
+    The arguments are the same as for uarray(...): nominal values, and
+    standard deviations.
 
     The returned matrix can be inverted, thanks to the fact that it is
     a unumpy.matrix object instead of a numpy.matrix one.
     """
 
-    return uarray(*args).view(matrix)
+    if std_devs is None:  # Obsolete, single tuple argument call
+        deprecation('umatrix() should now be called with two arguments.')
+        (nominal_values, std_devs) = nominal_values
+            
+    return uarray(nominal_values, std_devs).view(matrix)
 
 ###############################################################################
 
@@ -568,14 +594,11 @@ def define_vectorized_funcs():
         (f_name, 'arc'+f_name[1:])
         for f_name in ['acos', 'acosh', 'asin', 'atan', 'atan2', 'atanh'])
 
-    new_func_names = [
-        func_name_translations[function_name]
-        if function_name in func_name_translations
-        else function_name
-        for function_name in umath.many_scalar_to_scalar_funcs]
+    new_func_names = [func_name_translations.get(function_name, function_name)
+                      for function_name in umath.many_scalars_to_scalar_funcs]
         
-    for (function_name, unumpy_name) in \
-        zip(umath.many_scalar_to_scalar_funcs, new_func_names):
+    for (function_name, unumpy_name) in zip(
+        umath.many_scalars_to_scalar_funcs, new_func_names):
 
         # ! The newly defined functions (uncertainties.unumpy.cos, etc.)
         # do not behave exactly like their NumPy equivalent (numpy.cos,
